*  Program...........: TABENABLE.PRG
*  Author............: Jason M. Landry
*  Version...........: 0.50
*  Date..............: 5 April 1995
*  Company...........: Exchange Resources, Inc.
*                      5700 Green Circle Drive
*                      Minneapolis, MN 55444
*
*  This software is provided "as is" without
*  any warranty expressed or implied. In no event shall
*  Jason Landry, contributors, or distributors be liable
*  for any damages.
*
*  This software requires
*      TABS 2.0        | GENSCRNX driver by Steven Black
*      GENSCRNX 2.0,   | two public domain programs created by
*      3D 2.0,         | Ken Levy at the JPL in Pasadena, California.
*      FP 2.5 or better
*
*  WARNING!  Because this driver relies so heavily on the behavior
*  of the TABS driver, it is essential to use this driver ONLY
*  WITH TESTED RELEASES OF TABS.  Currently, it has been tested
*  with the TABS driver dated 3/23/95.  Use of any other TABS driver
*  could cause drought in California.
*
*  You are free to use this program as you see fit.  However, 
*  the distribution of modified source code is not allowed without the
*  expressed, written consent of the author.
*
*  Description.......: Modifies output from TABS driver and
*                      allows disabling/enabling any/all tabs.
*
*  Dependencies......: FP DOS/Windows 2.5 or greater
*                      GENSCRNX    2.0 or greater
*                      TABS Driver 2.6a dated 3/23/95 ONLY
*                      3D Driver   2.0 or greater
*
*  Calling Samples...: *:SCXDRV5 TABENABLE in screen setup screen 1
*  Parameter List....: -
*  Returns...........: -
*
*  Example calling structure
*        *:SCXDRV5 TABS
*        *:SCXDRV5 TABENABLE
*        *:SCXDRV5 INTL
*        *:SCXDRV5 3D
*        *:ALL 3D
*
*  tabenable Directives...:
*
*  In each screen's setup snippet,
*  *:tabenable expression
*   where expression is a valid FoxPro expression.  At runtime, this
*   expression will be evaluated and the tab screen enabled or disabled
*   based on the result.
*   Must return .T. | .F.

#DEFINE cnheader		1
#DEFINE cntext			5
#DEFINE cnline			6
#DEFINE cnbox			7
#DEFINE cnlist			11
#DEFINE cnpushbutton	12
#DEFINE cnradiobutton	13
#DEFINE cncheckbox		14
#DEFINE cnsayget		15
#DEFINE cnpopup			16
#DEFINE cnpicture		17
#DEFINE cnspinner		22
#DEFINE cninvbutton
#DEFINE cnexpression	0
#DEFINE cnprocedure		1
#DEFINE cngetcode		1
#DEFINE ccnull		CHR(0)
#DEFINE cnnormalwt	0
#DEFINE cnboldwt	1

#DEFINE cfssay 1
#DEFINE cfsget 2
#DEFINE cfsmsg 3
#DEFINE cfswhn 4
#DEFINE cfsval 5
#DEFINE cfspic 6
#DEFINE cfsokl 7

PRIVATE lctemp,lctabenable,i
GO TOP
IF scxcount=1
	RELEASE _tescreens
	PUBLIC _tescreens

	*- Make sure TABSTACK is OFF (Already processed, too late for us)
	IF UPPER(ALLTRIM(wordsearch('*-:TABSTACK','SETUPCODE')))='ON'
		WAIT WINDOW "WARNING:  TABSTACK MUST BE OFF" TIMEOUT 10
	ENDIF
	
	*- Get a list of all tab titles and put into _tescreens array
	lcalias = ALIAS()
	SELECT pjxdata
	lnrecno = RECNO()
	i=1
	SCAN FOR TYPE == 's'
		lctemp=ALLTRIM(name)
		DIMENSION _tescreens(i,2)
		SELECT 0
		USE (m.lctemp) AGAIN
		LOCATE FOR platform= m.platform_
		*- Make sure to skip those that are skipped by TABS
		IF wordsearch("*:TABSKIP ")=ccnull OR ALEN(_tescreens,1)=1
			m.lctablabel="["+ALLTRIM(wordsearch('*:TABLABEL'))+"]"
			IF lctablabel=ccnull
				lctablabel="["+ALLTRIM(name)+"]"
			ENDIF
			_tescreens(i,1)=lctablabel
			i=i+1
		ENDIF
		USE
		SELECT pjxdata
	ENDSCAN
	GO m.lnrecno
	SELECT ( m.lcalias)
	
	*- For now, always enable the first screen.
	*  m.lctabenable=wordsearch("*:tabenable")
	IF .T. or lctabenable=ccnull
		lctabenable=".t."
	ENDIF
	
	*- Make the assumption that TABS actually *has* been called.
	*- Disable normal pgup/pgdn behavior, but preserve existing
	*- ON KEY LABELS as the will not affect us.
	m.lctemp="*- Added by tabenable "+m.cr_lf+;
		"__curscn=1"+m.cr_lf+"__maxscn="+ALLTRIM(STR(m.screens))+m.cr_lf+;
		"IF EMPTY(ON('KEY','PGDN'))"+m.cr_lf+;
		"     ON KEY LABEL pgdn DO __pgvalid WITH 1"+m.cr_lf+;
		"ENDIF"+m.cr_lf+;
		"IF EMPTY(ON('KEY','PGUP'))"+m.cr_lf+;
		"   ON KEY LABEL pgup DO __pgvalid WITH -1"+m.cr_lf+;
		"ENDIF"+m.cr_lf+;
		"DIMENSION __tabenable("+ALLTRIM(STR(m.screens))+")"+m.cr_lf+;
		"__tabenable("+ALLTRIM(STR(scxcount))+")=EVAL('"+lctabenable+"')"

	*- Add enabled/disabled variable to setup code
	*- and fix GENSCRN #SECTION problems
	DO CASE
		CASE '#SECTION 2'$UPPER(setupcode) AND '#SECTION 1'$UPPER(setupcode)
			REPLACE setupcode WITH STUFF(setupcode,ATC('#SECTION 2',setupcode),0,m.cr_lf+lctemp+m.cr_lf)
		CASE '#SECTION 1'$UPPER(setupcode)
			REPLACE setupcode WITH setupcode+m.cr_lf+lctemp
		OTHERWISE
			REPLACE setupcode WITH "#SECTION 1"+m.cr_lf+lctemp+m.cr_lf+;
				"#SECTION 2"+m.cr_lf+setupcode+m.cr_lf
	ENDCASE

	*- Add pgup/pgdn procedure to skip disabled tabs
	REPLACE proccode WITH proccode+m.cr_lf+m.cr_lf+;
		"*- Added by tabenable"+m.cr_lf+;
		"FUNCTION __pgvalid"+m.cr_lf+;
		"PARAMETER val"+m.cr_lf+;
		"__curscn=__curscn+val"+m.cr_lf+;
		"IF __curscn<1"+m.cr_lf+"__curscn=__maxscn"+m.cr_lf+"ENDIF"+m.cr_lf+;
		"IF __curscn>__maxscn"+m.cr_lf+"__curscn=1"+m.cr_lf+"ENDIF"+m.cr_lf+;
		"DO WHILE !__tabenable(__curscn)"+m.cr_lf+;
		"__curscn=__curscn+val"+m.cr_lf+;
		"IF __curscn<1"+m.cr_lf+"__curscn=__maxscn"+m.cr_lf+"ENDIF"+m.cr_lf+;
		"IF __curscn>__maxscn"+m.cr_lf+"__curscn=1"+m.cr_lf+"ENDIF"+m.cr_lf+;
		"ENDDO"+m.cr_lf+;
		"jcScreenObj = 'm.___TABS' + REPLICATE(SUBSTR('ABCDEFGHIJKLMNOP',__curscn,1),2)"+m.cr_lf+;
		"_CUROBJ = OBJNUM( &jcScreenObj)"
	
	*- Restore any changed pgup/pgdn behavior
	REPLACE proccode WITH ;
		"IF '__pgvalid'$ON('KEY','PGUP')"+m.cr_lf+;
		"   ON KEY LABEL pgup"+m.cr_lf+;
		"ENDIF"+m.cr_lf+;
		"IF '__pgvalid'$ON('KEY','PGDN')"+m.cr_lf+;
		"   ON KEY LABEL pgdn"+m.cr_lf+;
		"ENDIF"+m.cr_lf+;
		proccode

ELSE
	*- We're on screen 2+
	m.lctabenable=wordsearch("*:tabenable")
	IF lctabenable=ccnull
		lctabenable=".t."
	ENDIF
	
	*- Add enabled/disabled variable to setup code
	*- and fix GENSCRN #SECTION problems
	m.lctemp="__tabenable("+ALLTRIM(STR(scxcount))+")=EVAL('"+lctabenable+"')"
	DO CASE
		CASE '#SECTION 2'$UPPER(setupcode) AND '#SECTION 1'$UPPER(setupcode)
			REPLACE setupcode WITH STUFF(setupcode,ATC('#SECTION 2',setupcode),0,m.cr_lf+lctemp+m.cr_lf)
		CASE '#SECTION 1'$UPPER(setupcode)
			REPLACE setupcode WITH setupcode+m.cr_lf+lctemp
		OTHERWISE
			REPLACE setupcode WITH "#SECTION 1"+m.cr_lf+lctemp+m.cr_lf+;
				"#SECTION 2"+m.cr_lf+setupcode+m.cr_lf
	ENDCASE
ENDIF

*- Bracket invisible buttons with enable/disable variable test
SCAN FOR objtype=20 AND name="m.___Tabs"
	=insif2("__tabenable("+ALLTRIM(STR(AT(RIGHT(name,1),"ABCDEFGHIJKLMNOPQRSTUVWXYZ")))+")")
ENDSCAN

*- Add an extra say for disabled tabs in disabled color for each
*- tab button in the screen.
FOR i=1 TO ALEN(_tescreens,1)
	LOCATE FOR objtype=cnsayget AND objcode=0 AND _tescreens(i,1)$expr AND !"*-:TABDONE"$comment
	IF FOUND()
		REPLACE comment WITH comment+m.cr_lf+"*-:TABDONE"
		=duprec()
		REPLACE penred WITH 128,penblue WITH 128, pengreen WITH 128
		SKIP -1
		=insif2("__tabenable("+ALLTRIM(STR(i))+")")
		SKIP
		=insif2("!__tabenable("+ALLTRIM(STR(i))+")")
	ENDIF
ENDFOR

*- We're done!
GO BOTTOM

IF m.scxcount=m.screens
	RELEASE _tescreens
ENDIF